//# GBTGOFiller: A filler for the GBT GO (user interface) FITS files.
//# Copyright (C) 2000,2001,2004
//# Associated Universities, Inc. Washington DC, USA.
//#
//# This library is free software; you can redistribute it and/or modify it
//# under the terms of the GNU Library General Public License as published by
//# the Free Software Foundation; either version 2 of the License, or (at your
//# option) any later version.
//#
//# This library is distributed in the hope that it will be useful, but WITHOUT
//# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
//# License for more details.
//#
//# You should have received a copy of the GNU Library General Public License
//# along with this library; if not, write to the Free Software Foundation,
//# Inc., 675 Massachusetts Ave, Cambridge, MA 02139, USA.
//#
//# Correspondence concerning AIPS++ should be addressed as follows:
//#        Internet email: aips2-request@nrao.edu.
//#        Postal address: AIPS++ Project Office
//#                        National Radio Astronomy Observatory
//#                        520 Edgemont Road
//#                        Charlottesville, VA 22903-2475 USA
//#
//#
//# $Id$

#ifndef NRAO_GBTGOFILLER_H
#define NRAO_GBTGOFILLER_H

#include <casa/aips.h>
#include <nrao/GBTFillers/GBTSimpleTable.h>

//# Forward declarations
namespace casa { //# NAMESPACE CASA - BEGIN
class Table;
class String;
} //# NAMESPACE CASA - END

#include <casa/namespace.h>

class GBTGOFile;

// <summary>
//  A filler for the GBT GO (user interface) FITS files.
// </summary>

// <use visibility=local>

// <reviewed reviewer="" date="yyyy/mm/dd" tests="" demos="">
// </reviewed>

// <prerequisite>
//   <li> Stuff
// </prerequisite>
//
// <etymology>
// This fills a subtable of a MS using the contents of the GO fits file.
// This fits file is generated by the GBT users interface and contains
// information relevent to how the user set up each scan.
// </etymology>
//
// <synopsis>
// </synopsis>
//
// <example>
// </example>
//
// <motivation>
// This file is not like the DAP files, it needs a separate filler.
// </motivation>
//
// <thrown>
//    <li>
//    <li>
// </thrown>
//

class GBTGOFiller
{
public:
    // makes a new NRAO_GBT_GLISH subtable if one is not already present in 
    // parent, otherwise it opens that one up as is. 
    GBTGOFiller(Table &parent);

    virtual ~GBTGOFiller();

    // fill the subtable using the supplied GBTGOFile.  
    // The scan number is used as an index column as is the time and interval.
    // The time should be the midpoint of the scan and the interval
    // is the duration of the scan.  A new row is added for each call
    // to this function.  trueScanNumber is used to compare the scan number
    // in the GO file with that actually being filled.  If the two differ,
    // a warning message is logged and the trueScanNumber is used.
    virtual void fill(const GBTGOFile &goFile, Double time, Double interval,
		      Int trueScanNumber);

    // flush the underlying simple table
    void flush() {itsTable->flush();}
private:
    GBTSimpleTable *itsTable;

    // unavailable, undefined
    GBTGOFiller();
    GBTGOFiller(const GBTGOFiller &);
    GBTGOFiller &operator=(const GBTGOFiller &);
};

#endif


